package Boot;

#   Copyright (c) 2001 International Business Machines

#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
 
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
 
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#   Donghwa John Kim <johkim@us.ibm.com>

use strict;
use Carp;
use Util::Log qw(:all);

#
#  Note: Order is important here.  It is the order in which the bootloaders
#  will be attempted.
#

use Boot::EFI;
use Boot::Elilo;
use Boot::Lilo;
use Boot::Grub;
use Boot::Palo;
use Boot::YaBoot;

use vars qw(@boottypes $VERSION);

$VERSION = sprintf("%d.%02d", q$Revision: 1.7 $ =~ /(\d+)\.(\d+)/);

sub install_config {
    my $config = shift;

    my @files = ();

    my %bootvars = $config->varlist("^(root|boot_|kernel)");
    my $complete = 0;

    foreach my $boottype (@boottypes) {
        my $boot = $boottype->new(%bootvars);
        verbose("Attempting to setup ". ref($boot));
        if($boot->footprint_loader()) {
            eval {
                $complete = $boot->install_config();
            };
        }
        if ($complete && !$@) {
            push @files, $boot->files(); #accessor method for the files modified
            last;
        }
    }
    
    unless ($complete) {
        croak("No support for bootloader");
    }
    return @files;
}

sub install_loader {
    my $config = shift;
    my %bootvars = $config->varlist("^(root|boot_|kernel)");
    my $complete = 0;
    
    foreach my $boottype (@boottypes) {
        my $boot = $boottype->new(%bootvars);
	### install_loader depends on the existence of both 
	### the bootloader and the config file
        if($boot->footprint_config() && $boot->footprint_loader()) {
            eval {
                $complete = $boot->install_loader();
            };
        }
        last if ($complete && !$@);
    }
    
    unless ($complete) {
        croak("No support for bootloader");
    }
    return 1;
}

=head1 NAME
Boot - The front end to setting up bootloaders in
                     System Configurator. 

=head1 SYNOPSIS
    use Boot;

    my  @filesmodified = Boot::setup_boot(%bootvars);

=head1 DESCRIPTION
Boot is the front end interface to setting up bootloaders.

It determines the type of bootloader (GRUB, LILO, YABOOT) installed on the system
and invoked its setup() method, which completes the configuration of the bootloader. 

For Intel x86 machines, if both GRUB and LILO are installed, GRUB has the precedence
over LILO, meaning that if configuration of GRUB succeeds LILO will not be configured.   
    
=head1 MODULE INTERFACE

To write a bootloader module which will plug into Boot, your
module must implement the following methods:

=over 4

=item new(%bootvariables)

The constructor expects a hash which contains bootloader configuration options as 
returned by the AppConfig method varlist("^(root|boot_|kernel)"). 
This must return an instance of the bootloader type.

=item footprint()

This instance method takes no arguements.  
It returns 1 if the type of bootloader is determined to be installed on the system, 
undef otherwise.

=item setup()

This instance method take no arguements. 
It completes the configuration of the bootloader for which footprint() succeeded.

=item files()

This instance method takes no arguements.  
It returns, in list context, a list of all files modified by the module.

=back

=head1 AUTHOR

  Donghwa John Kim <johkim@us.ibm.com>

=head1 SEE ALSO

L<Config>, L<Boot::Lilo>,
L<Boot::Grub>, L<Boot::YaBoot>

=cut

1;


















